@@ -2,10 +2,12 @@
 import socket, sys
 import asyncore
 from pysnmp.carrier import error
+from pysnmp import debug
 
 class AbstractSocketTransport(asyncore.dispatcher):
     sockFamily = sockType = None
     retryCount = 0; retryInterval = 0
+    bufferSize = 131070
     def __init__(self, sock=None, sockMap=None):
         if sock is None:
             if self.sockFamily is None:
@@ -20,6 +22,16 @@ class AbstractSocketTransport(asyncore.dispatcher):
                 sock = socket.socket(self.sockFamily, self.sockType)
             except socket.error:
                 raise error.CarrierError('socket() failed: %s' % sys.exc_info()[1])
+
+            try:
+                for b in socket.SO_RCVBUF, socket.SO_SNDBUF:
+                    bsize = sock.getsockopt(socket.SOL_SOCKET, b)
+                    if bsize < self.bufferSize:
+                        sock.setsockopt(socket.SOL_SOCKET, b, self.bufferSize)
+                        debug.logger & debug.flagIO and debug.logger('%s: socket %d buffer size increased from %d to %d for buffer %d' % (self.__class__.__name__, sock.fileno(), bsize, self.bufferSize, b))
+            except Exception:
+                debug.logger & debug.flagIO and debug.logger('%s: socket buffer size option mangling failure for buffer %d: %s' % (self.__class__.__name__, b, sys.exc_info()[1]))
+
         if sockMap is None:
             # The socket map is managed by the AsynsockDispatcher on
             # which this transport is registered, so this is a fake
