@@ -1,3 +1,4 @@
+# Implements I/O over asynchronous sockets
 from time import time
 from select import select
 from asyncore import socket_map
@@ -5,7 +6,6 @@ from pysnmp.carrier.base import AbstractTransportDispatcher
 from asyncore import poll
 
 class AsynsockDispatcher(AbstractTransportDispatcher):
-    """Implements I/O over asynchronous sockets"""
     def __init__(self):
         self.__sockMap = {} # use own map for MT safety
         self.timeout = 0.5
