@@ -1,9 +1,9 @@
-import time
+import sys
 from pyasn1.compat.octets import null
 from pysnmp.entity.rfc3413 import config
 from pysnmp.proto.proxy import rfc2576
 from pysnmp.proto.api import v2c
-from pysnmp.smi import error
+from pysnmp.proto import api, error
 from pysnmp import nextid
 from pysnmp import debug
 
@@ -14,7 +14,7 @@ class NotificationOriginator:
     def __init__(self, snmpContext):
         self.__pendingReqs = {}
         self.__pendingNotifications = {}
-        self.__context = snmpContext
+        self.snmpContext = snmpContext
 
     def processResponsePdu(
         self,
@@ -34,7 +34,7 @@ class NotificationOriginator:
         (cbFun, cbCtx) = cbInfo
         # 3.3.6d
         if sendPduHandle not in self.__pendingReqs:
-            raise error.PySnmpError('Missing sendPduHandle %s' % sendPduHandle)
+            raise error.ProtocolError('Missing sendPduHandle %s' % sendPduHandle)
 
         ( origTransportDomain,
           origTransportAddress,
@@ -67,6 +67,7 @@ class NotificationOriginator:
                     self._handleResponse(
                         metaSendPduHandle,
                         statusInformation['errorIndication'],
+                        0, 0, (),
                         cbFun,
                         cbCtx
                     )
@@ -76,23 +77,37 @@ class NotificationOriginator:
             timeoutInTicks = float(origTimeout)/100/snmpEngine.transportDispatcher.getTimerResolution()
         
             # 3.3.6a
-            sendPduHandle = snmpEngine.msgAndPduDsp.sendPdu(
-                snmpEngine,
-                origTransportDomain,
-                origTransportAddress,
-                origMessageProcessingModel,
-                origSecurityModel,
-                origSecurityName,
-                origSecurityLevel,
-                origContextEngineId,
-                origContextName,
-                origPduVersion,
-                origPdu,
-                1,                              # expectResponse
-                timeoutInTicks,
-                self.processResponsePdu,
-                (cbFun, cbCtx)
+            try:
+                sendPduHandle = snmpEngine.msgAndPduDsp.sendPdu(
+                    snmpEngine,
+                    origTransportDomain,
+                    origTransportAddress,
+                    origMessageProcessingModel,
+                    origSecurityModel,
+                    origSecurityName,
+                    origSecurityLevel,
+                    origContextEngineId,
+                    origContextName,
+                    origPduVersion,
+                    origPdu,
+                    1,                              # expectResponse
+                    timeoutInTicks,
+                    self.processResponsePdu,
+                    (cbFun, cbCtx)
                 )
+            except error.StatusInformation:
+                statusInformation = sys.exc_info()[1]
+                debug.logger & debug.flagApp and debug.logger('processResponsePdu: metaSendPduHandle %s: sendPdu() failed with %r ' % (metaSendPduHandle, statusInformation))
+                if not self.__pendingNotifications[metaSendPduHandle]:
+                    del self.__pendingNotifications[metaSendPduHandle]
+                    self._handleResponse(
+                        metaSendPduHandle,
+                        statusInformation['errorIndication'],
+                        0, 0, (),
+                        cbFun,
+                        cbCtx
+                    )
+                return
 
             self.__pendingNotifications[metaSendPduHandle] += 1
 
@@ -116,32 +131,50 @@ class NotificationOriginator:
                 origRetryCount,
                 origRetries + 1,
                 metaSendPduHandle
-                )
+            )
             return
 
         # 3.3.6c
         if not self.__pendingNotifications[metaSendPduHandle]:
             del self.__pendingNotifications[metaSendPduHandle]
-            self._handleResponse(metaSendPduHandle, None, cbFun, cbCtx)
+            pMod = api.protoModules[pduVersion]
+            self._handleResponse(metaSendPduHandle, None,
+                                 pMod.apiPDU.getErrorStatus(PDU),
+                                 pMod.apiPDU.getErrorIndex(PDU),
+                                 pMod.apiPDU.getVarBinds(PDU),            
+                                 cbFun, cbCtx)
 
-    def _handleResponse(
-        self,
-        sendRequestHandle,
-        errorIndication,
-        cbFun,
-        cbCtx):
-        cbFun(sendRequestHandle, errorIndication, cbCtx)
+    def _handleResponse(self,
+                        sendRequestHandle,
+                        errorIndication,
+                        errorStatus, errorIndex,
+                        varBinds,
+                        cbFun, cbCtx):
+        try:
+            # we need to pass response PDU information to user for INFORMs
+            cbFun(sendRequestHandle, errorIndication, 
+                  errorStatus, errorIndex, varBinds, cbCtx)
+        except TypeError:
+            # a backward compatible way of calling user function
+            cbFun(sendRequestHandle, errorIndication, cbCtx)
     
     def sendNotification(
         self,
         snmpEngine,
         notificationTarget,
         notificationName,
-        additionalVarBinds=None,
+        additionalVarBinds=(),
         cbFun=None,
         cbCtx=None,
-        contextName=null
+        contextName=null,
+        instanceIndex=None
         ):
+        debug.logger & debug.flagApp and debug.logger('sendNotification: notificationTarget %s, notificationName %s, additionalVarBinds %s, contextName "%s", instanceIndex %s' % (notificationTarget, notificationName, additionalVarBinds, contextName, instanceIndex))
+
+        if contextName:
+            __SnmpAdminString, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('SNMP-FRAMEWORK-MIB', 'SnmpAdminString')
+            contextName = __SnmpAdminString(contextName)
+ 
         # 3.3
         ( notifyTag,
           notifyType ) = config.getNotificationInfo(
@@ -151,11 +184,11 @@ class NotificationOriginator:
         metaSendPduHandle = getNextHandle()
 
         debug.logger & debug.flagApp and debug.logger('sendNotification: metaSendPduHandle %s, notifyTag %s, notifyType %s' % (metaSendPduHandle, notifyTag, notifyType))
-                
-        contextMibInstrumCtl = self.__context.getMibInstrum(
-            contextName
-            )
-        
+
+        contextMibInstrumCtl = self.snmpContext.getMibInstrum(contextName)
+       
+        additionalVarBinds = [  (v2c.ObjectIdentifier(x),y) for x,y in additionalVarBinds ]
+
         for targetAddrName in config.getTargetNames(snmpEngine, notifyTag):
             ( transportDomain,
               transportAddress,
@@ -180,42 +213,37 @@ class NotificationOriginator:
             varBinds = []
             
             # 3.3.2 & 3.3.3
-            sysUpTime, = contextMibInstrumCtl.mibBuilder.importSymbols(
-                '__SNMPv2-MIB', 'sysUpTime'
-                )
+            sysUpTime, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('__SNMPv2-MIB', 'sysUpTime')
 
-            if additionalVarBinds:
-                for varName, varVal in additionalVarBinds:
-                    if varName == sysUpTime.name:
-                        varBinds.append((varName, varVal))
-                        break
+            for varName, varVal in additionalVarBinds:
+                if varName == sysUpTime.name:
+                    varBinds.append((varName, varVal))
+                    break
             if not varBinds:
                 varBinds.append((sysUpTime.name,
                                  sysUpTime.syntax.clone())) # for actual value
 
-            snmpTrapOid, = contextMibInstrumCtl.mibBuilder.importSymbols(
-                '__SNMPv2-MIB', 'snmpTrapOID'
-                )
-            if notificationName:
+            snmpTrapOid, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('__SNMPv2-MIB', 'snmpTrapOID')
+            if len(notificationName) == 2:  # ('MIB', 'symbol')
+                notificationTypeObject, = contextMibInstrumCtl.mibBuilder.importSymbols(*notificationName)
+                varBinds.append((snmpTrapOid.name, v2c.ObjectIdentifier(notificationTypeObject.name)))
+                debug.logger & debug.flagApp and debug.logger('sendNotification: notification type object is %s' % notificationTypeObject)
+                for notificationObject in notificationTypeObject.getObjects():
+                    mibNode, = contextMibInstrumCtl.mibBuilder.importSymbols(*notificationObject)
+                    if instanceIndex:
+                        mibNode = mibNode.getNode(mibNode.name + instanceIndex)
+                    else:
+                        mibNode = mibNode.getNextNode(mibNode.name)
+                    varBinds.append((mibNode.name, mibNode.syntax))
+                    debug.logger & debug.flagApp and debug.logger('sendNotification: processed notification object %s, instance index %s, var-bind %s' % (notificationObject, instanceIndex is None and "<first>" or instanceIndex, mibNode))
+            elif notificationName:  # numeric OID
                 varBinds.append(
                     (snmpTrapOid.name,
                      snmpTrapOid.syntax.clone(notificationName))
-                    )
+                )
             else:
                 varBinds.append((snmpTrapOid.name, snmpTrapOid.syntax))
 
-# XXX it's still not clear how to instantiate OBJECTS clause
-#             # Get notification objects names
-#             for notificationObject in snmpTrapVal.getObjects():
-#                 mibNode, = contextMibInstrumCtl.mibBuilder.importSymbols(
-#                     *notificationObject
-#                     )
-#                 try:
-#                     objectInstance = mibNode.getNode(mibNode.name + (0,))
-#                 except error.SmiError:
-#                     return
-#                 varBinds.append((objectInstance.name, objectInstance.syntax))
-
             for varName, varVal in additionalVarBinds:
                 if varName in (sysUpTime.name, snmpTrapOid.name):
                     continue
@@ -224,7 +252,7 @@ class NotificationOriginator:
                         snmpEngine, securityModel, securityName,
                         securityLevel, 'notify', contextName, varName
                         )
-                except error.SmiError:
+                except error.StatusInformation:
                     debug.logger & debug.flagApp and debug.logger('sendNotification: OID %s not allowed for %s, droppping notification' % (varName, securityName))
                     return
                 else:
@@ -249,42 +277,70 @@ class NotificationOriginator:
             
             # 3.3.5
             if notifyType == 1:
-                snmpEngine.msgAndPduDsp.sendPdu(
-                    snmpEngine,
-                    transportDomain,
-                    transportAddress,
-                    messageProcessingModel,
-                    securityModel,
-                    securityName,
-                    securityLevel,
-                    self.__context.contextEngineId,
-                    contextName,
-                    pduVersion,
-                    pdu,
-                    None
+                try:
+                    snmpEngine.msgAndPduDsp.sendPdu(
+                        snmpEngine,
+                        transportDomain,
+                        transportAddress,
+                        messageProcessingModel,
+                        securityModel,
+                        securityName,
+                        securityLevel,
+                        self.snmpContext.contextEngineId,
+                        contextName,
+                        pduVersion,
+                        pdu,
+                        None
                     )
+                except error.StatusInformation:
+                    statusInformation = sys.exc_info()[1]
+                    debug.logger & debug.flagApp and debug.logger('sendReq: metaSendPduHandle %s: sendPdu() failed with %r' % (metaSendPduHandle, statusInformation))
+                    if not self.__pendingNotifications[metaSendPduHandle]:
+                        del self.__pendingNotifications[metaSendPduHandle]
+                        self._handleResponse(
+                            metaSendPduHandle,
+                            statusInformation['errorIndication'],
+                            0, 0, (),
+                            cbFun,
+                            cbCtx
+                        )
+                    return metaSendPduHandle
             else:
                 # Convert timeout in seconds into timeout in timer ticks
                 timeoutInTicks = float(timeout)/100/snmpEngine.transportDispatcher.getTimerResolution()
 
                 # 3.3.6a
-                sendPduHandle = snmpEngine.msgAndPduDsp.sendPdu(
-                    snmpEngine,
-                    transportDomain,
-                    transportAddress,
-                    messageProcessingModel,
-                    securityModel,
-                    securityName,
-                    securityLevel,
-                    self.__context.contextEngineId,
-                    contextName,
-                    pduVersion,
-                    pdu,
-                    1,                      # expectResponse
-                    timeoutInTicks,
-                    self.processResponsePdu,
-                    (cbFun, cbCtx)
+                try:
+                    sendPduHandle = snmpEngine.msgAndPduDsp.sendPdu(
+                        snmpEngine,
+                        transportDomain,
+                        transportAddress,
+                        messageProcessingModel,
+                        securityModel,
+                        securityName,
+                        securityLevel,
+                        self.snmpContext.contextEngineId,
+                        contextName,
+                        pduVersion,
+                        pdu,
+                        1,                      # expectResponse
+                        timeoutInTicks,
+                        self.processResponsePdu,
+                        (cbFun, cbCtx)
                     )
+                except error.StatusInformation:
+                    statusInformation = sys.exc_info()[1]
+                    debug.logger & debug.flagApp and debug.logger('sendReq: metaSendPduHandle %s: sendPdu() failed with %r' % (metaSendPduHandle, statusInformation))
+                    if not self.__pendingNotifications[metaSendPduHandle]:
+                        del self.__pendingNotifications[metaSendPduHandle]
+                        self._handleResponse(
+                            metaSendPduHandle,
+                            statusInformation['errorIndication'],
+                            0, 0, (),
+                            cbFun,
+                            cbCtx
+                        )
+                    return metaSendPduHandle
 
                 debug.logger & debug.flagApp and debug.logger('sendNotification: metaSendPduHandle %s, sendPduHandle %s, timeout %d' % (metaSendPduHandle, sendPduHandle, timeout))
                 
@@ -296,7 +352,7 @@ class NotificationOriginator:
                     securityModel,
                     securityName,
                     securityLevel,
-                    self.__context.contextEngineId,
+                    self.snmpContext.contextEngineId,
                     contextName,
                     pduVersion,
                     pdu,
