@@ -55,7 +55,6 @@ _snmpErrors = {
 class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
     messageProcessingModelID = univ.Integer(3) # SNMPv3
     snmpMsgSpec = SNMPv3Message
-    _scopedPDU = ScopedPDU()
     _emptyStr = univ.OctetString('')
     _msgFlags = {
         0: univ.OctetString('\x00'),
@@ -67,6 +66,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
         }
     def __init__(self):
         AbstractMessageProcessingModel.__init__(self)
+        self.__scopedPDU = ScopedPDU()
         self.__engineIDs = {}
         self.__engineIDsExpQueue = {}
         self.__expirationTimer = 0
@@ -119,10 +119,10 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
         if not contextName:
             contextName = self._emptyStr
 
-        debug.logger & debug.flagMP and debug.logger('prepareOutgoingMessage: using contextEngineId %s, contextName %s' % (repr(contextEngineId), contextName))
+        debug.logger & debug.flagMP and debug.logger('prepareOutgoingMessage: using contextEngineId %r, contextName %r' % (contextEngineId, contextName))
         
         # 7.1.6
-        scopedPDU = self._scopedPDU
+        scopedPDU = self.__scopedPDU
         scopedPDU.setComponentByPosition(0, contextEngineId)
         scopedPDU.setComponentByPosition(1, contextName)
         scopedPDU.setComponentByPosition(2)
@@ -196,7 +196,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
                     2, self._msgFlags[msgFlags & 0xfc], verifyConstraints=False
                     )
                 # XXX
-                scopedPDU = self._scopedPDU
+                scopedPDU = self.__scopedPDU
                 scopedPDU.setComponentByPosition(
                     0, self._emptyStr, verifyConstraints=False
                     )
@@ -214,7 +214,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
             else:
                 securityEngineID = peerSnmpEngineData['securityEngineID']
 
-        debug.logger & debug.flagMP and debug.logger('prepareOutgoingMessage: securityEngineID %s' % repr(securityEngineID))
+        debug.logger & debug.flagMP and debug.logger('prepareOutgoingMessage: securityModel %r, securityEngineID %r, securityName %r, securityLevel %r' % (securityModel, securityEngineID, securityName, securityLevel))
              
         # 7.1.9.b
         ( securityParameters,
@@ -266,7 +266,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
         contextName,
         pduVersion,
         pdu,
-        maxSizeResponseScopedPDU,
+        maxSizeResponseScopedPDU,  # n.b. not actually used here
         stateReference,
         statusInformation
         ):
@@ -349,10 +349,10 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
         if not contextName:
             contextName = self._emptyStr
 
-        debug.logger & debug.flagMP and debug.logger('prepareResponseMessage: using contextEngineId %s, contextName %s' % (repr(contextEngineId), contextName))
+        debug.logger & debug.flagMP and debug.logger('prepareResponseMessage: using contextEngineId %r, contextName %r' % (contextEngineId, contextName))
 
         # 7.1.6
-        scopedPDU = self._scopedPDU
+        scopedPDU = self.__scopedPDU
         scopedPDU.setComponentByPosition(0, contextEngineId)
         scopedPDU.setComponentByPosition(1, contextName)
         scopedPDU.setComponentByPosition(2)
@@ -417,6 +417,8 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
                 errorIndication = errind.unsupportedSecurityModel
                 )
 
+        debug.logger & debug.flagMP and debug.logger('prepareResponseMessage: securityModel %r, securityEngineID %r, securityName %r, securityLevel %r' % (securityModel, snmpEngineID, securityName, securityLevel))
+
         # 7.1.8a
         try:
             ( securityParameters,
@@ -459,6 +461,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
                 wholeMsg, asn1Spec=self._snmpMsgSpec
                 )
         except PyAsn1Error:
+            debug.logger & debug.flagMP and debug.logger('prepareDataElements: %s' % (sys.exc_info()[1],))
             snmpInASNParseErrs, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('__SNMPv2-MIB', 'snmpInASNParseErrs')
             snmpInASNParseErrs.syntax = snmpInASNParseErrs.syntax + 1
             raise error.StatusInformation(
@@ -524,7 +527,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
                 )
             debug.logger & debug.flagMP and debug.logger('prepareDataElements: SM succeeded')
         except error.StatusInformation:
-            statusInformation = sys.exc_info()[1]
+            statusInformation, origTraceback = sys.exc_info()[1:3]
             debug.logger & debug.flagMP and debug.logger('prepareDataElements: SM failed, statusInformation %s' % statusInformation)
             if 'errorIndication' in statusInformation:
                 # 7.2.6a
@@ -585,25 +588,36 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
                     
                     debug.logger & debug.flagMP and debug.logger('prepareDataElements: error reported')
             # 7.2.6b
-            raise statusInformation
+            if sys.version_info[0] <= 2:
+                raise statusInformation
+            else:
+                try:
+                    raise statusInformation.with_traceback(origTraceback)
+                finally:
+                    # Break cycle between locals and traceback object	
+                    # (seems to be irrelevant on Py3 but just in case)
+                    del origTraceback
         else:
             # Sniff for engineIDs
             k = (transportDomain, transportAddress)
             if k not in self.__engineIDs:
-                contextEngineId, contextName, pdu = scopedPDU
-                
-                self.__engineIDs[k] = {
-                    'securityEngineID': securityEngineID,
-                    'contextEngineId': contextEngineId,
-                    'contextName': contextName
-                    }
+                contextEngineId, contextName, pdus = scopedPDU
+                pdu = pdus.getComponent()
+                # Here we assume that authentic/default EngineIDs
+                # come only in the course of engine-to-engine communication.
+                if pdu.tagSet in rfc3411.internalClassPDUs:
+                    self.__engineIDs[k] = {
+                        'securityEngineID': securityEngineID,
+                        'contextEngineId': contextEngineId,
+                        'contextName': contextName
+                        }
 
-                expireAt = self.__expirationTimer + 300
-                if expireAt not in self.__engineIDsExpQueue:
-                    self.__engineIDsExpQueue[expireAt] = []
-                self.__engineIDsExpQueue[expireAt].append(k)
+                    expireAt = int(self.__expirationTimer + 300 / snmpEngine.transportDispatcher.getTimerResolution())
+                    if expireAt not in self.__engineIDsExpQueue:
+                        self.__engineIDsExpQueue[expireAt] = []
+                    self.__engineIDsExpQueue[expireAt].append(k)
                     
-                debug.logger & debug.flagMP and debug.logger('prepareDataElements: cache securityEngineID %s for %s %s' % (repr(securityEngineID), transportDomain, transportAddress))
+                    debug.logger & debug.flagMP and debug.logger('prepareDataElements: cache securityEngineID %r for %r %r' % (securityEngineID, transportDomain, transportAddress))
 
         snmpEngineID, = snmpEngine.msgAndPduDsp.mibInstrumController.mibBuilder.importSymbols('__SNMP-FRAMEWORK-MIB', 'snmpEngineID')
         snmpEngineID = snmpEngineID.syntax
@@ -773,7 +787,7 @@ class SnmpV3MessageProcessingModel(AbstractMessageProcessingModel):
         if self.__expirationTimer in self.__engineIDsExpQueue:
             for engineKey in self.__engineIDsExpQueue[self.__expirationTimer]:
                 del self.__engineIDs[engineKey]
-                debug.logger & debug.flagMP and debug.logger('__expireEnginesInfo: expiring %s' % (engineKey,))
+                debug.logger & debug.flagMP and debug.logger('__expireEnginesInfo: expiring %r' % (engineKey,))
             del self.__engineIDsExpQueue[self.__expirationTimer]
         self.__expirationTimer = self.__expirationTimer + 1
         
